==> emelfm2-0.1.7.tar.gz: emelfm2-0.1.7/plugins/e2p_du.c <==
/* $Id: e2p_du.c 1861 2006-04-25 08:27:44Z tpgww $
 *
 * Copyright (C) 2003-2005 tooar <tooar@gmx.net>
 * Portions copyright (C) 1999 Michael Clark
 *
 * This file is part of emelFM2.
 * emelFM2 is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * emelFM2 is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with emelFM2; see the file GPL. If not, contact the Free Software
 * Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

#include "emelfm2.h"
//#include <unistd.h>
#include <langinfo.h>
#include <dirent.h>
#include <string.h>
#include "e2_plugins.h"

// binary factor
#define BFAC 1024.0
#define BFAC2 1048576.0
#define BFAC3 1073741824.0

typedef struct
{
	guint64 total;
	guint64 files;
	guint64 dirs;
	gboolean hidden;
} E2_Du;

/**
 * @brief update du counters
 * This is a callback for the treewalk function
 * @param local_name path of item reported by the walker, localised string
 * @param statbuf pointer to struct stat with data about @a local_name
 * @param status code from the walker, indicating what type of item is being reported
 * @param user_data pointer to data struct with total, files, dirs counters
 * @return E2TW_CONTINUE always
 **/
static E2_TwResult _e2p_du_twcb (const gchar *local_name,
	const struct stat *statbuf, E2_TwStatus status, E2_Du *user_data)
{
	gchar *ptr = strrchr (local_name, G_DIR_SEPARATOR);
	if (ptr == NULL)
	{
		if (local_name[0] == '.')
			user_data->hidden = TRUE;
	}
	else if (*(ptr+1) == '.')
		user_data->hidden = TRUE;

	switch (status)
	{
		case E2TW_F:	//not directory or link
		case E2TW_SL:	//symbolic link
		case E2TW_SLN:	//symbolic link naming non-existing file
			user_data->files++;
			user_data->total += statbuf->st_size;
			break;
		case E2TW_DL:	//directory, not opened due to tree-depth limit
		case E2TW_DM:	//directory, not opened due to different file system
		{
			gchar *utf = F_FILENAME_FROM_LOCALE ((gchar *)local_name);
			gchar *msg = g_strdup_printf ("%s %s", _("Cannot open directory"), utf);
			e2_output_print_error (msg, TRUE);
			F_FREE (utf);
		}
		break;
		case E2TW_DNR:	//unreadable directory
		{
			gchar *utf = F_FILENAME_FROM_LOCALE ((gchar *)local_name);
			gchar *msg = g_strdup_printf ("%s %s - ", _("Cannot open directory"), utf);
			e2_output_print (curr_tab, msg, NULL, FALSE, E2_ERRORTAGS, NULL);
			e2_output_print_errno ();	//only E2TW_DNR?
			F_FREE (utf);
			g_free (msg);
		}
		case E2TW_D:	//directory
			user_data->dirs++;
			user_data->total += statbuf->st_size;
			break;
//		case E2TW_DP:	//directory, all subdirs have been visited
//		case E2TW_NS:	//un-statable item FIXME warning needed
		default:
			break;
	}
	return E2TW_CONTINUE;
}
/**
 * @brief thread function to iterate over active pane file list to get sizes
 *
 * @return NULL
 **/
static gpointer _e2p_du_all (gpointer data)
{
	FileInfo *info;
	guint64 total, files, dirs;
	gboolean hashidden;
	GString *text;
	GList *base, *tmp;
	gchar *local = F_FILENAME_TO_LOCALE (curr_view->dir);
	gchar *local_path;
	//all counters initialized to 0
	E2_Du *cbdata = g_malloc0 (sizeof (E2_Du));
	base = tmp = e2_fileview_get_selected_local (curr_view);
	for (; tmp != NULL; tmp = tmp->next)
	{
		info = tmp->data;
//		printd (DEBUG, "fn: %s", info->filename);
		local_path = e2_utils_strcat (local, info->filename);
		e2_fs_tw (local_path, _e2p_du_twcb, cbdata, -1, E2TW_PHYS);
		g_free (local_path);
	}
	F_FREE (local);
	total = cbdata->total;
	files = cbdata->files;
	dirs = cbdata->dirs;
	hashidden = cbdata->hidden;
	g_free (cbdata);
	g_list_free (base);

	static gchar big[3] = { '1', ',', '\0' };
	gchar *comma = nl_langinfo (THOUSEP);
	if (comma != NULL && *comma != '\0')
		big[1] = *comma;
	text = g_string_new(_("total size: "));
	gint fwidth;

	if (total < BFAC)
	{
		gchar *b = _("bytes");
		if (total < 1000)
			g_string_append_printf (text, "%llu %s", total, b);
		else
		{
			total -= 1000;
	    	g_string_append_printf (text, "%s%03llu %s", big, total, b);
		}
	}
	else if (total < BFAC2)
	{
		gchar *kb = _("kilobytes");
		fwidth = (total < 10 * BFAC) ? 3 : 2;

		if ((total/BFAC) < 1000)
			g_string_append_printf(text, "%.*f %s", fwidth, (total / BFAC), kb);
		else
		{
			total -= (1000 * BFAC);
	    	g_string_append_printf (text, "%s%04.1f %s", big, (total / BFAC), kb);
		}
	}
	else if (total < BFAC3)
	{
		gchar *mb = _("Megabytes");
		fwidth = (total < 10 * BFAC2) ? 3 : 1;

		if ((total/BFAC2) < 1000)
			g_string_append_printf(text, "%.*f %s", fwidth, (total / BFAC2), mb);
		else
		{
			total -= (1000 * BFAC2);
			g_string_append_printf (text, "%s%04.1f %s", big, (total / BFAC2), mb);
		}
	}
	else
	{
		gchar *gb = _("gigabytes");
		fwidth = (total < 10 * BFAC3) ? 3 : 1;

		if ((total/BFAC3) < 1000)
			g_string_append_printf(text, "%.*f %s", fwidth, (total / BFAC3), gb);
		else
		{
			total -= (1000 * BFAC3);
			g_string_append_printf (text, "%s%04.1f %s", big, (total / BFAC3), gb);
		}
	}

	gchar *filetext = (files == 1) ? _("file") : _("files");
	gchar *dirtext = (dirs == 1) ? _("directory") : _("directories");
	g_string_append_printf (text, "\n%s %llu %s %s %llu %s",
		_("in"), files, filetext, _("and"), dirs, dirtext);
	if (hashidden)
		g_string_append_printf (text, " %s\n", _("(one or more are hidden)"));
	else
		text = g_string_append_c (text, '\n');
	e2_output_print_default (text->str);
	e2_output_print (&app.tab, _("----end-of-output----\n"), NULL, FALSE, "small", "grey", NULL);
	g_string_free (text, TRUE);
	return NULL;
}

/**
 * @brief create thread to iterate over active pane selected items to check size
 *
 * @param action_data ptr to data assigned when action struct created at session start
 * @param rt_data ptr to data sent to this command, NULL to act on selection
 * @param widget optional pointer to widget where the action was initiated, generally NULL
 *
 * @return TRUE if action completed successfully, else FALSE
 */
static gboolean _e2p_du (gpointer action_data, gpointer rt_data, GtkWidget *widget)
{
	g_thread_create (_e2p_du_all, NULL, FALSE, NULL);
	return TRUE;
}

//aname must be confined to this module
static gchar *aname;
/**
 * @brief plugin initialization function, called by main program
 *
 * @param p ptr to plugin data struct
 *
 * @return TRUE if the initialization succeeds, else FALSE
 **/
gboolean
init_plugin (Plugin *p)
{
#define ANAME "du"
	aname = _("du");

	p->signature = ANAME VERSION;
	p->menu_name = _("_Disk usage");
	p->description = _("Calculate the 'apparent' disk usage of selected item(s)");
	p->icon = "plugin_"ANAME"_"E2IP".png";  //icon file pathname

	if (p->plugin_cb == NULL)
	{
		p->plugin_cb = _e2p_du;  //set plugins menu item function
		gchar *action_name = g_strconcat (_A(5),".",aname,NULL);
		e2_plugins_action_register (action_name, E2_ACTION_TYPE_ITEM, _e2p_du,
		  NULL, FALSE, 0, NULL);
		return TRUE;
	}
	return FALSE;
}
/**
 * @brief cleanup transient things for this plugin
 *
 * @param p pointer to data struct for the plugin
 *
 * @return TRUE if all cleanups were completed
 **/
gboolean clean_plugin (Plugin *p)
{
  gchar *action_name = g_strconcat (_A(5),".",aname,NULL);
  gboolean ret = e2_plugins_action_unregister (action_name);
  g_free (action_name);
  return ret;
}

==> emelfm2-0.1.8.tar.gz: emelfm2-0.1.8/plugins/e2p_du.c <==
/* $Id: e2p_du.c 2053 2006-07-06 23:47:23Z tpgww $
 *
 * Copyright (C) 2003-2005 tooar <tooar@gmx.net>
 * Portions copyright (C) 1999 Michael Clark
 *
 * This file is part of emelFM2.
 * emelFM2 is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * emelFM2 is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with emelFM2; see the file GPL. If not, contact the Free Software
 * Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

#include "emelfm2.h"
//#include <unistd.h>
#include <langinfo.h>
#include <dirent.h>
#include <string.h>
#include "e2_plugins.h"

// binary factor
#define BFAC 1024.0
#define BFAC2 1048576.0
#define BFAC3 1073741824.0

typedef struct
{
	guint64 total;
	guint64 files;
	guint64 dirs;
	gboolean hidden;
} E2_Du;

/**
 * @brief update du counters
 * This is a callback for the treewalk function
 * @param local_name path of item reported by the walker, localised string
 * @param statbuf pointer to struct stat with data about @a local_name
 * @param status code from the walker, indicating what type of item is being reported
 * @param user_data pointer to data struct with total, files, dirs counters
 * @return E2TW_CONTINUE always
 **/
static E2_TwResult _e2p_du_twcb (const gchar *local_name,
	const struct stat *statbuf, E2_TwStatus status, E2_Du *user_data)
{
	gchar *ptr = strrchr (local_name, G_DIR_SEPARATOR);
	if (ptr == NULL)
	{
		if (local_name[0] == '.')
			user_data->hidden = TRUE;
	}
	else if (*(ptr+1) == '.')
		user_data->hidden = TRUE;

	switch (status)
	{
		case E2TW_F:	//not directory or link
		case E2TW_SL:	//symbolic link
		case E2TW_SLN:	//symbolic link naming non-existing file
			user_data->files++;
			user_data->total += statbuf->st_size;
			break;
		case E2TW_DL:	//directory, not opened due to tree-depth limit
		case E2TW_DM:	//directory, not opened due to different file system
		{
			gchar *utf = F_DISPLAYNAME_FROM_LOCALE ((gchar *)local_name);
			gchar *msg = g_strdup_printf ("%s %s", _("Cannot open directory"), utf);
			e2_output_print_error (msg, TRUE);
			F_FREE (utf);
		}
		break;
		case E2TW_DNR:	//unreadable directory
		{
			gchar *utf = F_DISPLAYNAME_FROM_LOCALE ((gchar *)local_name);
			gchar *msg = g_strdup_printf ("%s %s - ", _("Cannot open directory"), utf);
			e2_output_print (&app.tab, msg, NULL, FALSE, E2_ERRORTAGS, NULL);
			e2_output_print_strerrno ();	//only E2TW_DNR?
			F_FREE (utf);
			g_free (msg);
		}
		case E2TW_D:	//directory
			user_data->dirs++;
			user_data->total += statbuf->st_size;
			break;
//		case E2TW_DP:	//directory, all subdirs have been visited
//		case E2TW_NS:	//un-statable item FIXME warning needed
		default:
			break;
	}
	return E2TW_CONTINUE;
}
/**
 * @brief thread function to iterate over active pane file list to get sizes
 * @param showtab pointer to tab which was current when the thread was initiated
 * @return NULL
 **/
//FIXME with E2_ASYNC, curr_view etc is not relevant
static gpointer _e2p_du_all (E2_OutputTabRuntime *starttab)
{
	TABLOG (starttab)
	FileInfo *info;
	guint64 total, files, dirs;
	gboolean hashidden;
	GString *text;
	GList *base, *tmp;
	gchar *local = F_FILENAME_TO_LOCALE (curr_view->dir);
	gchar *local_path;

	//all counters initialized to 0
	E2_Du *cbdata = ALLOCATE0 (E2_Du);
	CHECKALLOCATEDWARN (cbdata, return NULL;)
	base = tmp = e2_fileview_get_selected_local (curr_view);
	for (; tmp != NULL; tmp = tmp->next)
	{
		info = tmp->data;
//		printd (DEBUG, "fn: %s", info->filename);
		local_path = e2_utils_strcat (local, info->filename);
		e2_fs_tw (local_path, _e2p_du_twcb, cbdata, -1, E2TW_PHYS);
		g_free (local_path);
	}
	F_FREE (local);
	total = cbdata->total;
	files = cbdata->files;
	dirs = cbdata->dirs;
	hashidden = cbdata->hidden;
	DEALLOCATE (E2_Du, cbdata);
	g_list_free (base);

	static gchar big[3] = { '1', ',', '\0' };
	gchar *comma = nl_langinfo (THOUSEP);
	if (comma != NULL && *comma != '\0')
		big[1] = *comma;
	text = g_string_new(_("total size: "));
	gint fwidth;

	if (total < BFAC)
	{
		gchar *b = _("bytes");
		if (total < 1000)
			g_string_append_printf (text, "%llu %s", total, b);
		else
		{
			total -= 1000;
	    	g_string_append_printf (text, "%s%03llu %s", big, total, b);
		}
	}
	else if (total < BFAC2)
	{
		gchar *kb = _("kilobytes");
		fwidth = (total < 10 * BFAC) ? 3 : 2;

		if ((total/BFAC) < 1000)
			g_string_append_printf(text, "%.*f %s", fwidth, (total / BFAC), kb);
		else
		{
			total -= (1000 * BFAC);
	    	g_string_append_printf (text, "%s%04.1f %s", big, (total / BFAC), kb);
		}
	}
	else if (total < BFAC3)
	{
		gchar *mb = _("Megabytes");
		fwidth = (total < 10 * BFAC2) ? 3 : 1;

		if ((total/BFAC2) < 1000)
			g_string_append_printf(text, "%.*f %s", fwidth, (total / BFAC2), mb);
		else
		{
			total -= (1000 * BFAC2);
			g_string_append_printf (text, "%s%04.1f %s", big, (total / BFAC2), mb);
		}
	}
	else
	{
		gchar *gb = _("gigabytes");
		fwidth = (total < 10 * BFAC3) ? 3 : 1;

		if ((total/BFAC3) < 1000)
			g_string_append_printf(text, "%.*f %s", fwidth, (total / BFAC3), gb);
		else
		{
			total -= (1000 * BFAC3);
			g_string_append_printf (text, "%s%04.1f %s", big, (total / BFAC3), gb);
		}
	}

	gchar *filetext = (files == 1) ? _("file") : _("files");
	gchar *dirtext = (dirs == 1) ? _("directory") : _("directories");
	g_string_append_printf (text, "\n%s %llu %s %s %llu %s",
		_("in"), files, filetext, _("and"), dirs, dirtext);
	if (hashidden)
		g_string_append_printf (text, " %s\n", _("(one or more are hidden)"));
	else
		text = g_string_append_c (text, '\n');

	gdk_threads_enter ();
	e2_output_print_same (text->str);
	gdk_threads_leave ();
	E2_OutputTabRuntime *tab = (starttab == curr_tab) ? &app.tab : starttab;
	gdk_threads_enter ();
	e2_output_print_end (tab, FALSE);
	gdk_threads_leave ();
	g_string_free (text, TRUE);
	return NULL;
}

/**
 * @brief create thread to iterate over active pane selected items to check size
 *
 * @param from the button, menu item etc which was activated
 * @param art action runtime data
 *
 * @return TRUE
 */
static gboolean _e2p_du (gpointer from, E2_ActionRuntime *art)
{
	g_thread_create ((GThreadFunc)_e2p_du_all, curr_tab, FALSE, NULL);
	return TRUE;
}

//aname must be confined to this module
static gchar *aname;
/**
 * @brief plugin initialization function, called by main program
 *
 * @param p ptr to plugin data struct
 *
 * @return TRUE if the initialization succeeds, else FALSE
 **/
gboolean
init_plugin (Plugin *p)
{
#define ANAME "du"
	aname = _("du");

	p->signature = ANAME VERSION;
	p->menu_name = _("_Disk usage");
	p->description = _("Calculate the 'apparent' disk usage of selected item(s)");
	p->icon = "plugin_"ANAME"_"E2IP".png";  //icon file pathname

	if (p->action == NULL)
	{
		gchar *action_name = g_strconcat (_A(5),".",aname,NULL);
		p->action = e2_plugins_action_register
		(action_name, E2_ACTION_TYPE_ITEM, _e2p_du, NULL, FALSE, 0, NULL);
		return TRUE;
	}
	return FALSE;
}
/**
 * @brief cleanup transient things for this plugin
 *
 * @param p pointer to data struct for the plugin
 *
 * @return TRUE if all cleanups were completed
 **/
gboolean clean_plugin (Plugin *p)
{
  gchar *action_name = g_strconcat (_A(5),".",aname,NULL);
  gboolean ret = e2_plugins_action_unregister (action_name);
  g_free (action_name);
  return ret;
}
